                                                                
;----------------------------------------------------------------------
;
;   Filename:           decode_int_transmit.asm
;   Date:               June 22nd, 2004
;   File Version:       1.0
;   Assembled using:    MPASM 03.30 Released
;
;   Author:             Jay Summet
;
;
;----------------------------------------------------------------------
;
;   Files required:
;                       p12f675.inc
;
;
;----------------------------------------------------------------------
;
;   Program Description 
;
;   When GP3 is pushed, this program reads 65000 A/D sensor readings, and
;   computes a threshold value. It then starts to look for a 60hz serial signal,
;   and syncs up with it and starts to decode bits..
;
;   Then it starts to record bits, and search for valid datawords (e.g. checksum
;   is valid and the "ID" portion of the payload is 0x15). When it finds a valid dataword,
;   it broadcasts it over the radio.
;
;	It uses an interrupt driven transmitter, so that it can continue to look for a
;   new set of valid data while transmitting the last decoded set of data.
;


;----------------------------------------------------------------------

    list        p=12f675        ; list directive to define processor
    #include    <p12f675.inc>   ; processor specific variable definitions

    errorlevel  -302            ; suppress message 302 from list file

; ---------------------------------------------------------------------
; Configuration Bits (Section 9.1 Configuration Bits)
; ---------------------------------------------------------------------
;
; Data Memory Code Protection bit:
; _CPD_ON = Enabled
; _CPD_OFF = Disabledb
;
; Program Memory Code protection:
; _CP_ON = Enabled
; _CP_OFF = : Disabled
;
; Brown-out Detection Enable bit:
; _BODEN_ON = Enabled
; _BODEN_OFF = Disabled
;
; GP3/MCLR pin function select:
; _MCLRE_ON = GP3/MCLR pin function is /MCLR
; _MCLRE_OFF = GP3/MCLR pin function is digital I/O, 
;              /MCLR internally tied to Vdd
;
; Power-up Timer Enable bit:
; _PWRTE_ON = Enabled
; _PWRTE_OFF = Disabled
;
; Watchdog Timer Enable bit:
; _WDT_ON = Enabled
; _WDT_OFF = Disabled
;
; Oscillator Selction bits:
; _EXTRC_OSC_NOCLKOUT = CLKOUT function on GP4 pin, RC on GP5 pin.
; _EXTRC_OSC_CLKOUT = I/O function on GP4 pin, RC on GP5 pin.
; _INTRC_OSC_CLKOUT =  Internal oscillator, CLKOUT function on GP4 pin,
;                      I/O function on GP5 pin.
; _INTRC_OSC_NOCLKOUT = Internal oscillator, I/O function on GP4 and GP5 pins.
; _EC_OSC = I/O function on GP4 pin, CLKIN on GP5 pin.
; _HS_OSC = High speed crystal/resonator on GP4 and GP5 pins.
; _XT_OSC = Crystal/resonator on GP4 and GP5 pins.
; _LP_OSC = Low power crystal on GP4 and GP5 pins.
;
;
; ---------------------------------------------------------------------

    __CONFIG    _CPD_OFF & _CP_OFF & _BODEN_OFF & _MCLRE_OFF & _PWRTE_OFF & _WDT_OFF & _INTRC_OSC_NOCLKOUT

;----------------------------------------------------------------------
; Variables  (Section 2.2 Data Memory Organization)
;----------------------------------------------------------------------

    ; Data Memory Organization (Section 2.2)
    ;
    ; Register locations 0x20 to 0x5F (64 bytes) are General Purpose 
    ; registers, implemented as static RAM and are mapped across both
    ; banks. 

    cblock  0x20

        w_temp                      ; used for context saving 
        status_temp                 ; used for context saving

        TEMP                        ; General Purpose Temporary register


;; Used by FindThreshold... Count's reused by autosync
        CountLow
        CountHigh

	HighValue
	LowValue
 	ThresholdValue

	;; Used multiple places whenever a value is sensed.
	SensedValue
	

;; Used by auto-sync:
   LastState		; Stores the last state (1 bit, high/low)
   DriftCount		; Used to determine when to drift upwards....
   MBLAvHigh
   MBLAvLow
   MBLAvCount

   MBLHigh			; High & Low of 16 bit variable, storing
   MBLLow			; the value of the smallest edge-to-edge seen so far.
   SensedValues		; Stores the thresholded bits from last 8 checks.


  	BSCLow			; Used to determine when a new sample should be taken...(BitSampleCount Low/High)
	BSCHigh			;


   SampledBits1	; Stores the last 24 sampled bits, (oldest) 3 <- 2 <- 1 (Newest)
   SampledBits2
   SampledBits3
    ;; Add a few more for testing purposes...
   SampledBits4
   SampledBits5
   SampledBits6


	;; Used by DECODE to store the results of the checkbits...
   CheckBits 

   LowByteToSend
   HighByteToSend

; for testing
	ProbeCounter

   TotalBitsLow
   TotalBitsHigh

   GoodOnesLow
   GoodOnesHigh

    ;; Used various places to find the average of multiple input samples.
	Accumulator1
	Accumulator2
	CountTemp


        CSR0                        ; TX buffer shift register
        CSR1
        CSR2
        CSR3
        CSR4

  ;; Not sending these 4 bytes to save time...
  ;      CSR5
  ;      CSR6
  ;      CSR7
  ;      CSR8
        Count
        Count2
        BitCount
        TimeHi
        TimeLo

        FuncBits                    ; Function Bits


;; Extra Stuff for the Interrupt Driven Transmission!
		TX_STATUS			; 0 = set means currently transmitting
							; 1 = Bit we are currently transmitting.

 	 	TX_STATE			; Stores the state we are in...
		TX_COUNTER			; Used to count a lot of things.
        TX_BIT_COUNTER		; Stores the number of bits we have sent.
		TX_BYTE_TEMP		; Stores the each byte as we send it.
        TX_FSR				; Stores the value of FSR when we are not in the ISR.

 

    endc

;; MACRO: Swap W and a register with no extra registers...
;; used by TX module
SWAPWF	MACRO REG
		xorwf REG,F
		xorwf REG,W
		xorwf REG,F
		ENDM

;----------------------------------------------------------------------
; Defines
;----------------------------------------------------------------------

    ; Set up GPIO Port (Section 3.0)
    ; Function of GPIO pins depend on:
    ;   Configuration Bits (CONFIG) (Section 9.1)
    ;   Weak Pull-up Register (WPU) (Section 3.2.1)
    ;   Interrupt-on-change GPIO Register (IOCB) (Section 3.2.2)
    ;   Option Register (OPTION_REG) (Register 4-1)
    ;   TIMER1 Control Register (T1CON) (Register 5-1)
    ;   Comparator Control Register (CMCON) (Section 6.0)
    ;   A/D Control Register (ADCON0) (Section 7.0) (PIC12F675 Only)

#define POT0    GPIO, 0             ; (Analog Input) Potentiometer GP0
#define POT1    GPIO, 1             ; (Analog Input) Potentiometer GP1
#define TXD     GPIO, 2             ; (Output) Transmit Data
#define PB3     GPIO, 3             ; (Input Only) Push button switch GP3
#define PB4     GPIO, 4             ; (Input) Push button switch GP4
#define RFENA   GPIO, 5             ; (Output) RF Enable

    ; Define for TRISIO Register (Section 3.1)
    ;
    ; GPIO is an 6-bit wide, bi-directional port. The corresponding data
    ; direction register is TRISIO. Setting a TRISIO bit (= 1) will make
    ; the corresponding GPIO pin and input. Clearing a TRISIO bit (= 0)
    ; will make the corresponding GPIO pin an output. The exception is
    ; GP3, which is input only and its TRIS bit will always read as a '1'.

    ; GPIO Pins = xx543210
#define GPTRIS  B'00011011'

    ; delays/timings for TX module...

#define TGUARD  D'46'               ; 46 X TE
#define PREAMB  D'16'               ; Preamble length = 16 pulses
#define TSYNC   D'10'				; 10 x TE


;----------------------------------------------------------------------
; Program Memory
;----------------------------------------------------------------------

    ; Program Memory Organization (Section 2.1)

        ORG     0x000               ; RESET Vector

        nop                         ; for ICD use
        goto    INITALIZE           ; goto INITALIZE


        ORG     0x004               ; Interrupt Vector

        movwf   w_temp              ; save W register
        swapf   STATUS, W           ; swap status to be saved into W
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
        movwf   status_temp         ; save STATUS register

;----------------------------------------
; Interrupt Service Routine (ISR)
;
; Description: 
;   
;----------------------------------------
; Interrupt-on-change (Section 3.2.2 and 9.4.3)
;
; An input change on GPIO change sets the GPIF bit. The interrupt can
; be enabled/disabled be setting/clearing the GPIE bit. Individual pins
; are configured through the IOC register (see INITIALIZATION below).
;
; Clear the IOC interrupt by:
;  a) Any read or write of GPIO. This will end the mismatch condition.
;  b) Clear the flag bit GPIF

        movfw   GPIO                ; read GPIO
        bcf     INTCON, GPIF
;----------------------------------------
;; If this is a Timer0 interrupt, call the
;; Interrupt Driven TX transmission handler...
     ;movf INTCON,W
     ;andlw 0x20
     ;btfsc STATUS,Z				;; IS T0IE set?
     ;goto RETURN_FROM_ISR		;; no, continue.
     movf INTCON,W				;; Check T0IF
     andlw 0x04		
     btfsc STATUS,Z				;; If T0IF is set, handle it!
      goto RETURN_FROM_ISR

	

	;; If we are inactive we set this to zero...
    clrw  ;; Load zero into w, to put into TMR0 unless we are active.
    btfsc TX_STATUS,0
    movlw D'215'		;; If we are active, set TMR0 to 210 = 49 x 8 cycles = 400
    movwf TMR0

     btfsc TX_STATUS,0			;; Check to see if we are active.
     call TX_INTERRUPT			;; Go do the real work!


     bsf     STATUS, RP0         ; ---- Select Bank 1 -----
     bcf INTCON,T0IF			;; Clear the TMR0 interrupt flag.
     bcf     STATUS, RP0         ; ---- Select Bank 0 -----
 ;----------------------------------------
RETURN_FROM_ISR
        swapf   status_temp, W      ; swap status_temp into W, sets bank to original state
        movwf   STATUS              ; restore STATUS register
        swapf   w_temp, F
        swapf   w_temp, W           ; restore W register

        retfie


;----------------------------------------------------------------------
; Subroutine DATA_EEPROM_READ
;
; Description: To read an EEPROM data memory location, the address is
;   written to the EEADR register and set control bit RD (EECON1<0>) to
;   initiate a read. Data is available in the EEDATA register the next
;   clock cycle.
;
; Constants: none
;   
; Global Variables: none
;   
; Initialization: W contains EEPROM address to be read
;   
; Output: W contains EEPROM data
;   
;----------------------------------------------------------------------

DATA_EEPROM_READ

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        movwf   EEADR               ; move EEPROM address in W to EEADR
        bsf     EECON1, RD          ; initiate EEPROM read
        movf    EEDATA, W           ; move data to W

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----

        return


;----------------------------------------------------------------------
; Subroutine DATA_EEPROM_WRITE
;
; Description: To write an EEPROM data memory location, the address is
;   written to the EEADR register, data to the EEDATA register, then
;   execute a required sequence of instructions.
;   
; CAUTION: Interrupts are disable during this subroutine
;
; Constants: none
;   
; Global Variables: none
;   
; Initialization: Address = EEADR, Data = EEDATA
;   
; Output: none
;   
;----------------------------------------------------------------------

DATA_EEPROM_WRITE

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        bsf     EECON1, WREN        ; EEPROM Write Enable: allow write cycles
        bcf     INTCON, GIE         ; disable global interrupts
                                    ; *** required sequence, do not alter ***
        movlw   0x55
        movwf   EECON2
        movlw   0xAA
        movwf   EECON2
        bsf     EECON1, WR          ; initiate EEPROM write
                                    ; *** end required sequence ***
        bsf     INTCON, GIE         ; enable global interrupts
        bcf     EECON1, WREN        ; EEPROM Write Enable: inhibit write cycles

      
;; Wait for the write to finish!
DEW_WAIT_LOOP
        btfsc   EECON1,WR			; Querry the "I'm done" bit... 
		goto DEW_WAIT_LOOP			; loop until it's cleared by hardware... (yes, it's a waste of cycles, but hey...)

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
		
        return


;----------------------------------------------------------------------
; Subroutine READ_ANALOG_AN0; READ_ANALOG_AN1
;
; Description: Read analog channel 0 (AN0) or 1 (AN1).
;   
; Constants: none
;   
; Global Variables: none
;   
; Initialization: none
;   
; Output: ADRESH and ADRESL contain 10-bit A/D result justified 
;   according to ADCON0, ADFM bit.
;   W contains the value upon return...
;   
;----------------------------------------------------------------------

READ_ANALOG_AN0

        bcf     ADCON0, CHS1        ; select analog channel AN0
        bcf     ADCON0, CHS0

        goto    READ_ANALOG

READ_ANALOG_AN1

        bcf     ADCON0, CHS1        ; select analog channel AN1
        bsf     ADCON0, CHS0

READ_ANALOG

        bsf     ADCON0, ADON        ; Turn on ADC module

        ; After selecting a new channel, allow for sufficent sample time.
        ; The amount of sample time depends on the charging time of the
        ; internal charge holding capacitor (Section 7.2).

        movlw   D'9'                ; At 4 MHz, a 22us delay ; JWS modified 6 to 9 for extra safety
        movwf   TEMP                ; (22us = 2us + 6 * 3us + 1us)
        decfsz  TEMP, F
        goto    $-1

        bsf     ADCON0, GO          ; start A/D conversion

        btfsc   ADCON0, GO          ; has A/D conversion completed?
        goto    $-1

        bcf     ADCON0, ADON        ; Turn off ADC module (consumes no operating current)

;; Read analog result into W.
        ;bsf     STATUS, RP0     ; ----- Select Bank 1 -----
        movfw   ADRESH         ; ADRESH Result
        ;bcf     STATUS, RP0     ; ----- Select Bank 0 -----

        return


;----------------------------------------------------------------------
; Subroutine FIND_THRESHOLD
;
; Description: Read analog channel 0 (AN0) for 4 seconds and determines
;				a threshold value. (stores HighValue, LowValue, subtracts
;				them to find a range, divides the range by 2, then adds
;				it to the low value.
;   
; Constants: none
;   
; Global Variables: HighValue - Highest value seen so far.
;; 			LowValue  - Lowest value seen so far.
;;			SensedValue - temp variable holding the sensed value.
;;			Count1		- temp counter variable.
;;			Count2		- temp counter vairable. 
;;			ThresholdValue - Threshold Value
;   
; Initialization: none
;   
; Output: ThresholdValue 
;   
;----------------------------------------------------------------------
FIND_THRESHOLD
 ; We are going to cycle 255x255=65025 times... (about 5 seconds @ 4Mhz)

	movlw   D'255'		; load countHigh with 255...
	movwf   CountHigh

	movlw	0x00		; Start with HighVal at zero
	movwf	HighValue	; move to HighValue
	movlw	0xFF		; Start with LowValue at 0xff (255)
	movwf   LowValue	; Move to LowValue

FIND_THRESHOLD_MAIN_LOOP 

 movlw d'255'  ; Average 255 samples...  
  movwf CountTemp
 clrf Accumulator1
 clrf Accumulator2

FT_AVERAGE_LOOP
    call READ_ANALOG_AN0
	movwf  SensedValue	; Store in SensedValue...

	addwf Accumulator1,f
	btfss STATUS,C
	goto FT_SLC2
	incf Accumulator2,f	; inc accumulator2
FT_SLC2
	decfsz	CountTemp,f
	goto FT_AVERAGE_LOOP

;; We copy only the value from Accumulator2, essentially dividing by 256...

	movfw Accumulator2
	movwf  SensedValue	; Store in SensedValue...

	movfw LowValue
	subwf SensedValue,0	; Subtract (SensedValue - LowValue)
	btfsc	STATUS,Z	; If they are the same, jump to TEST_HIGH_VALUE
	goto FT_TEST_HIGH_VALUE



				; If  (sensedValue - LowValue) > 0, do nothing for lowvalue and
	btfsc STATUS,C		; If LowValue is bigger than sensedValue, skip.
	goto FT_TEST_HIGH_VALUE     ; go test HighValue


				; replace previous lowValue with SensedValue.	
	movfw SensedValue	; Put SensedValue into Low Value!
	movwf LowValue

	




FT_TEST_HIGH_VALUE

	movfw SensedValue	; Move sensed value into W.
	subwf HighValue,0	;  HighValue - SensedValue, store in W
	btfsc	STATUS,Z	; If they are the same, jump to LOOP_GAIN
	goto FT_LOOP_AGAIN

        btfsc STATUS,C          ; If sensedValue is bigger than HighValue, skip
	goto FT_LOOP_AGAIN
	
	;; If HighValue is too low, move sensed value into it...
	movfw SensedValue
	movwf HighValue


FT_LOOP_AGAIN
 

	decfsz CountHigh,f		; Decrement countHigh by one, skip if zero.. 
	 goto FIND_THRESHOLD_MAIN_LOOP      ; otherwise, just start the loop again...

				; If it's zero, we are done.



;; Find the threshold...
	movfw LowValue		; 
	subwf HighValue,0	; Subtract HighValue - LowValue, result in W
	movwf ThresholdValue	; Move to Threshold Value
	bcf	STATUS,C	; Clear the carry bit for our divide to work!
	rrf ThresholdValue,0	; Divide by 2, leave in W
	addwf LowValue,0	; Add LowValue, leave in W
	movwf ThresholdValue	; save in ThresholdValue.

  ;; We are all done.
  RETURN
;----------------------------------------------------------------------
; Subroutine: WaitxTE
;   
; Description: 
;   
; Constants: 
;   
; Global Variables: 
;   
; Initialization: W x 400us length of delay required
;   
; Output: CountLow, CountHigh   as decrementing counters
;   
;----------------------------------------------------------------------

WaitxTE
        movwf   CountHigh      ; [1]

waitxlp            
        movlw   D'79'       ; [1] 	
        movwf   CountLow       ; [1]

wait400lp
        nop                 ; [1]
        nop                 ; [1]
        decfsz  CountLow,F     ; [1]
         goto    wait400lp  ; [2]
;                          --------
;                         79 x 5 = 395us

        decfsz  CountHigh,F    ; [1]
         goto    waitxlp    ; [2]

        retlw   0           ; [2]

;                       total 2 (call) + W x (395 + 5) + 2 (return)
;                       w = 1 -> 406us @4MHz
;                       w = 2 -> 806us @4MHz


;----------------------------------------------------------------------
; Initialize PICmicro (PIC12F675)
;----------------------------------------------------------------------

INITALIZE

; Disable global interrupts during initialization

        bcf     INTCON, GIE         ; disable global interrupts


;----------------------------------------
; Calibrating the Internal Oscillator (Section 9.2.5.1)
; Oscillator Calibration Register (Section 2.2.2.7)
;
; A calibration instruction is programmed into the last location of
; program memory. This instruction is a RETLW XX, where the literal is
; the calibration value. The literal is placed in the OSCCAL register
; to set the calibration of the internal oscillator.

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        call    0x3FF               ; retrieve factory calibration value
        movwf   OSCCAL              ; update register with factory cal value

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----


;----------------------------------------
; GPIO Port (Section 3.0)
;
; Store GPTRIS value defined above into the TRISIO direction register

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        movlw   GPTRIS
        movwf   TRISIO              ; Write to TRISIO register

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----


;----------------------------------------
; Comparator Module (Section 6.0)
;
; The PIC12F629/675 devices have one analog comparator. The inputs to
; the comparator are multiplexed with the GP0 and GP1 pins. There is 
; an on-chip Comparator Voltage Reference that can also be applied to
; an input of the comparator. In addition, GP2 can be configured as
; the comparator output. The Comparator Control Register (CMCON)
; contains bits to control the comparator. The Voltage Reference
; Control Register (VRCON) controls the voltage reference module.


        ; Comparator Configuration (Figure 6-2)
;        bcf     CMCON, CINV         ; Comparator Output Inversion: not inverted
;        bcf     CMCON, COUT         ; Comparator Output bit: Vin+ < Vin-
;        bcf     CMCON, CIS          ; Comparator Input Switch: Vin- connectos to Cin-

        ; CM2:CM0 = 111 - Comparator Off (lowest power)
        bsf     CMCON, CM2          ; Comparator Mode bit 2
        bsf     CMCON, CM1          ; Comparator Mode bit 1
        bsf     CMCON, CM0          ; Comparator Mode bit 0

        ; VRCON (Register 6-2)
        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        bcf     VRCON, VREN         ; CVref circuit: powered down, no Idd drain

;        bcf     VRCON, VRR          ; CVref Range Selection: High Range

;        bcf     VRCON, VR3          ; CVref value selection bit 3
;        bcf     VRCON, VR2          ; CVref value selection bit 2
;        bcf     VRCON, VR1          ; CVref value selection bit 1
;        bcf     VRCON, VR0          ; CVref value selection bit 0

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----


;----------------------------------------
; Analog-to-Digital Converter (A/D) Module (Section 7.0) (PIC12F675 Only)
;
; The analog-to-digital converter (A/D) allows conversion of an analog
; input signal to a 10-bit binary representation of that signal. The
; PIC12F675 has four analog inputs multiplexed into one sample and hold
; circuit. There are two registers to control the functions of the A/D
; module:
;   A/D Control Register (ADCON0)
;   Analog Select Register (ANSEL)
;
; Note: When using GPIO pins as analog inputs, ensure the TRISIO register
;       bits are set (= 1) for input.

        bcf     ADCON0, ADFM        ; A/D Result Formed: left justified
        bcf     ADCON0, VCFG        ; Voltage Reference: Vdd
        bcf     ADCON0, ADON        ; ADC is shut-off and consumes no operating current

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        ; select A/D Conversion Clock Source: Fosc/8
        bcf     ANSEL, ADCS2        ; A/D Conversion Clock Select bit 2
        bcf     ANSEL, ADCS1        ; A/D Conversion Clock Select bit 1
        bsf     ANSEL, ADCS0        ; A/D Conversion Clock Select bit 0

        ; select GPIO pins that will be analog inputs: GP0/AN0, GP1/AN1
        bcf     ANSEL, ANS3         ; Analog Select GP4/AN3: digital I/O
        bcf     ANSEL, ANS2         ; Analog Select GP2/AN2: digital I/O
        bsf     ANSEL, ANS1         ; Analog Select GP1/AN1: analog input
        bsf     ANSEL, ANS0         ; Analog Select GP0/AN0: analog input

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----


;----------------------------------------
; TIMER1 Module with Gate Control (Section 5.0)
;
; The TIMER1 Control Register (T1CON) is used to enable/disable TIMER1
; and select various features of the TIMER1 module.

        bsf     T1CON, TMR1ON       ; TIMER1: Enabled

        bcf     T1CON, TMR1CS       ; TIMER1 Clock Source Select: Internal Clock (Fosc/4)

        bcf     T1CON, NOT_T1SYNC   ; TIMER1 External Clock Input Sync Control: Syncronize external clock input

        ; T1OSCEN only if INTOSC without CLKOUT oscillator is active, else ignored
        bcf     T1CON, T1OSCEN      ; LP Oscillator Enable Control: LP oscillator off

        ; TIMER1 Input Prescale Select: 1:8  (1 inc for every 8 instructions, largest we can get)
        bsf     T1CON, T1CKPS1      ; TIMER1 Input Clock Prescale Select bit 1
        bsf     T1CON, T1CKPS0      ; TIMER1 Input Clock Prescale Select bit 0

        ; TMR1GE only if TMR1ON = 1, else ignored
        bcf     T1CON, TMR1GE       ; TIMER1 Gate Enable: on


;----------------------------------------
; Weak Pull-up Register (WPU) (Section 3.2.1)
;
; Each of the GPIO pins, except GP3, has an individually configurable
; weak internal pull-up. Control bits WPUx enable or disable each 
; pull-up. Refer to Register 3-1. Each weak pull-up is automatically
; turned off when the port pin is configured as an output. The pull-ups
; are disabled on a Power-on Reset by the NOT_GPPU bit (see OPTION_REG below).

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

;     GPIO Pins = xx54x210
        movlw   B'00010000'         ; GP4 pull-up enabled
        movwf   WPU

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----

;
; TIMER0 is used by the Interrupt driven transmiter.

;----------------------------------------
; OPTION Register (OPTION_REG) (Section 2.2.2.2)
; TIMER0 Module (Section 4.0)
;
; The OPTION_REG contains control bits to configure:
;   Weak pull-ups on GPIO (see also WPU Register above)
;   External GP2/INT interrupt
;   TMR0
;   TMR0/WDT prescaler

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        bcf     OPTION_REG, NOT_GPPU  ; GPIO pull-ups: enabled

        bsf     OPTION_REG, INTEDG  ; Interrupt Edge: on rising edge of GP2/INT pin

        bcf     OPTION_REG, T0CS    ; TMR0 Clock Source: internal instruction cycle (CLKOUT)
        bcf     OPTION_REG, T0SE    ; TMR0 Source Edge: increment low-to-high transition on GP2/T0CKI pin

        bcf     OPTION_REG, PSA     ; Prescaler Assignment: assigned to TIMER0

        ; TMR0 Prescaler Rate: 1:8
        bcf     OPTION_REG, PS2     ; Prescaler Rate Select bit 2
        bsf     OPTION_REG, PS1     ; Prescaler Rate Select bit 1
        bcf     OPTION_REG, PS0     ; Prescaler Rate Select bit 0

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----


;----------------------------------------
; Interrupt-on-Change Register (IOCB) (Section 3.2.2)
;
; Each of the GPIO pins is individually configurable as an interrupt-
; on-change pin. Control bits IOCBx enable or disable the interrupt 
; function for each pin. Refer to Register 3-2. The interrupt-on-change
; is disabled on a Power-on Reset.
;
; Note: Global interrupt enables (GIE and GPIE) must be enabled for
;       individual interrupts to be recognized.

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

    ; GPIO Pins = xx543210
;        movlw   b'00011000'
        movlw   b'00001000'	    
        movwf   IOCB                ; Interrupt-on-change enabled: GP3 ONLY

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----


;----------------------------------------
; Peripheral Interrupt Enable Register (PIE1) (Section 2.2.2.4)
;
; The PIE1 register contains peripheral interrupt enable bits.
;
; Note: The PEIE bit (INTCON<6>) must be set to enable any 
;       peripheral interrupt.

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        bcf     PIE1, EEIE          ; EE Write Complete Interrupt: disabled
        bcf     PIE1, ADIE          ; A/D Converter Interrupt (PIC12F675 Only): disabled
        bcf     PIE1, CMIE          ; Comparator Interrupt: disabled
        bcf     PIE1, TMR1IE        ; TMR1 Overflow Interrupt: disabled

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----


;----------------------------------------
; Interrupt Control Register (INTCON) (Section 2.2.2.3)
;
; The INTCON register bcontains enable and disable flag bits for TMR0
; register overflow, GPIO port change and external GP2/INT pin 
; interrupts.

        bcf     INTCON, PEIE        ; disable Peripheral Interrupt Enable bit
        bsf     INTCON, T0IE        ; enable TMR0 Overflow Interrupt Enable bit (Interrupt driven transmitter)
        bcf     INTCON, INTE        ; disable GP2/INT External Interrupt Enable bit
        bsf     INTCON, GPIE        ; enable Port Change Interrupt Enable bit

;        bcf     INTCON, GIE         ; disable global interrupts


;----------------------------------------------------------------------
;----------------------------------------------------------------------
; Main Program
;----------------------------------------------------------------------
;----------------------------------------------------------------------

MAIN

        clrf TX_STATE			; Start with TX_State at zero...
        bcf     RFENA           ; Disable Transmitter
        bsf     INTCON, GIE     ; enable global interrupts


; We want to sleep immediately (ie do nothing) and only wake up when GP3 is pushed.
	bcf RFENA 		; disable the Transmitter

   ;; For testing, initalize some stuff...
    clrf TotalBitsLow
    clrf TotalBitsHigh
    clrf GoodOnesLow
    clrf GoodOnesHigh
;	sleep			; (when we wake up, we'll continue to START_ON_GP3_PRESS)

;----------------------------------
; When somebody pressed GP3, we go!
;----------------------------------

START_ON_GP3_PRESS
 ; enable the transmitter (only for diagnostic purposes..)
	bsf RFENA		;enable transmitter (and associated LED ;>)
	






 call FIND_THRESHOLD	; Determine the threshold values for 4 seconds.
 bcf RFENA

  clrf ProbeCounter


  


	
	goto RESET_AUTOSYNC ; Start the AUTOSYNC_LOOP at the "reset-everything" stage...


AUTOSYNC_LOOP


        btfsc   PB4             ; Push Button GP4 depressed?
         goto   SPB2            ; no, jump over

 ;; FOR TESTING: write the current values of MBLHigh and MBLLow to EEPROM:
SAVE_DEBUG_STATE_AND_SLEEP


; Save our "findings" to EEPROM so I can pull them off with the programmer for debuging purposes...

   bsf     STATUS, RP0         ; ---- Select Bank 1 -----
        movlw   0x00              ; EEPROM Address = 0x00
        movwf   EEADR
        movfw   LowValue           ; Save value
        movwf   EEDATA
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
        call    DATA_EEPROM_WRITE   ; write first byte to EEPROM


        bsf     STATUS, RP0         ; ---- Select Bank 1 -----
        movlw   0x01              ; EEPROM Address 
        movwf   EEADR
        movfw   ThresholdValue           ; Save value
        movwf   EEDATA
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
        call    DATA_EEPROM_WRITE   ; write byte to EEPROM

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----
        movlw   0x02              ; EEPROM Address 
        movwf   EEADR
        movfw   HighValue           ; Save value
        movwf   EEDATA
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
        call    DATA_EEPROM_WRITE   ; write byte to EEPROM

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----
        movlw   0x04              ; EEPROM Address 
        movwf   EEADR
        movfw   MBLHigh           ; Save value
        movwf   EEDATA
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
        call    DATA_EEPROM_WRITE   ; write byte to EEPROM

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----
        movlw   0x05              ; EEPROM Address 
        movwf   EEADR
        movfw   MBLLow           ; Save value
        movwf   EEDATA
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
        call    DATA_EEPROM_WRITE   ; write byte to EEPROM

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----
        movlw   0x06             ; EEPROM Address 
        movwf   EEADR
        movlw   0xFF           ; Save value
        movwf   EEDATA
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
        call    DATA_EEPROM_WRITE   ; write byte to EEPROM

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----
        movlw   0x07             ; EEPROM Address 
        movwf   EEADR
        movfw   ProbeCounter           ; Save value
        movwf   EEDATA
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
        call    DATA_EEPROM_WRITE   ; write byte to EEPROM

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----
        movlw   0x09             ; EEPROM Address 
        movwf   EEADR
        movfw   SensedValues           ; Save value
        movwf   EEDATA
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
        call    DATA_EEPROM_WRITE   ; write byte to EEPROM


        bsf     STATUS, RP0         ; ---- Select Bank 1 -----
        movlw   0x0a             ; EEPROM Address 
        movwf   EEADR
        movfw  	SampledBits3           ; Save value
        movwf   EEDATA
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
        call    DATA_EEPROM_WRITE   ; write byte to EEPROM

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----
        movlw   0x0b             ; EEPROM Address 
        movwf   EEADR
        movfw   SampledBits2           ; Save value
        movwf   EEDATA
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
        call    DATA_EEPROM_WRITE   ; write byte to EEPROM

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----
        movlw   0x0c             ; EEPROM Address 
        movwf   EEADR
        movfw   SampledBits1           ; Save value
        movwf   EEDATA
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
        call    DATA_EEPROM_WRITE   ; write byte to EEPROM

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----
        movlw   0x1a             ; EEPROM Address 
        movwf   EEADR
        movfw  	SampledBits6           ; Save value
        movwf   EEDATA
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
        call    DATA_EEPROM_WRITE   ; write byte to EEPROM

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----
        movlw   0x1b             ; EEPROM Address 
        movwf   EEADR
        movfw   SampledBits5           ; Save value
        movwf   EEDATA
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
        call    DATA_EEPROM_WRITE   ; write byte to EEPROM

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----
        movlw   0x1c             ; EEPROM Address 
        movwf   EEADR
        movfw   SampledBits4           ; Save value
        movwf   EEDATA
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
        call    DATA_EEPROM_WRITE   ; write byte to EEPROM

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----
        movlw   0x10             ; EEPROM Address 
        movwf   EEADR
        movfw   BSCHigh           ; Save value
        movwf   EEDATA
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
        call    DATA_EEPROM_WRITE   ; write byte to EEPROM

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----
        movlw   0x11             ; EEPROM Address 
        movwf   EEADR
        movfw   BSCLow           ; Save value
        movwf   EEDATA
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
        call    DATA_EEPROM_WRITE   ; write byte to EEPROM

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----
        movlw   0x20             ; EEPROM Address 
        movwf   EEADR
        movfw   GoodOnesHigh           ; Save value
        movwf   EEDATA
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
        call    DATA_EEPROM_WRITE   ; write byte to EEPROM

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----
        movlw   0x21             ; EEPROM Address 
        movwf   EEADR
        movfw   GoodOnesLow          ; Save value
        movwf   EEDATA
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
        call    DATA_EEPROM_WRITE   ; write byte to EEPROM

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----
        movlw   0x30             ; EEPROM Address 
        movwf   EEADR
        movfw   TotalBitsHigh           ; Save value
        movwf   EEDATA
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
        call    DATA_EEPROM_WRITE   ; write byte to EEPROM

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----
        movlw   0x31             ; EEPROM Address 
        movwf   EEADR
        movfw   TotalBitsLow           ; Save value
        movwf   EEDATA
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
        call    DATA_EEPROM_WRITE   ; write byte to EEPROM


       ;; Now that we've written all the values, reset!
        goto START_ON_GP3_PRESS

   
 ;; END FOR TESTING

SPB2
;; Every time: Check the Timer. If it's about to overflow, it means that we have not seen an edge
;; 				in the input for WAY too long, reset everything and start looking again.
  	movlw 0xFF
  	xorwf TMR1H,w			; Xor Timer1High with FF. (store in W as we only care about STATUS.Z)
  	btfss STATUS,Z		; If it's not FF, we jump to the next item, (increment MBL by one...)
  	goto AS_INC_MBL
	goto RESET_AUTOSYNC


AS_INC_MBL

;; Used to be called in an attempt to audosync to signals other than 60hz.
;;;; call BIT_LENGTH_DRIFT_UP

	goto AS_NEW_SENSE
						
RESET_AUTOSYNC


    clrf TMR1L			; Reset the timer.
    clrf TMR1H			;

	clrf SampledBits1
    clrf SampledBits2
	clrf SampledBits3	;; The previous three could be removed to save space....


	movlw d'1'
    movwf DriftCount		;; Reset DriftCount
						;; As well as the MBLHigh and MBLLow bits...
	movlw 0xDF			;; Leave some room to catch timer1 before an overflow..
	movwf MBLHigh		
	movwf MBLLow		;; Ideally the low bytes would be FF, but this saves us an instruction word

						;; Set the sample Bit count timer to something high so we hopefully won't sample
	movwf BSCHigh		;; Anything until we get our first edge.
	movwf BSCLow

	movwf CountHigh	
	movwf CountLow
    bsf    LastState,0	;; By default we "reset" to a last state of one. This could potentially be
						;; Removed to save memory. It's also a problem if the current state ISN'T a zero, and we have to
						;; recover from a potentially very short (but incorrect) inital MBL setting.
						;; so our "drift up" on MBL needs to be moderately fast, but not too fast...



AS_NEW_SENSE
 

;; For testing.... Show light when synced.
  ; bsf RFENA ; turn on transmitter...
  ; movlw 0x09
  ; subwf MBLHigh,w	; Subtract MBLHigh - 0x09
  ; btfsc STATUS,C
  ;  	bcf RFENA 		; disable the Transmitter




  movlw d'16'  ; Average 16 samples...  
  movwf CountTemp
  clrf Accumulator1
  clrf Accumulator2

AS_AVERAGE_LOOP
    call READ_ANALOG_AN0
	movwf  SensedValue	; Store in SensedValue...

	addwf Accumulator1,f
	btfss STATUS,C
	goto AS_SLC2
	incf Accumulator2,f	; inc accumulator2
AS_SLC2
	decfsz	CountTemp,f
	goto AS_AVERAGE_LOOP

;; SHIFT 4 bits to divide by 16!
   movlw d'4'
   movwf CountTemp
AS_SHIFT_LOOP
   rrf Accumulator2,f
   rrf Accumulator1,f
   decfsz CountTemp,f
   goto AS_SHIFT_LOOP

   ;;Sample against the Threshold value.
	movfw Accumulator1

	subwf ThresholdValue,w 	;(Threshold - W, store in W)

  ;; Invert the bit for our sanity!
; JWS - FOR TESTING - Changed the hardware so it inverts High/Low for Dim/Bright, so
; no longer need to invert the bit!
;  rlf TEMP,f
;  movlw 0xff
;  xorwf TEMP,f
;  rrf TEMP,f

 

	rlf   SensedValues,f		; Save the result (in carry bit) to SensedValues.

    

;;;----------------------------------------------------------
;; We check to see if we want to save the same value into our 
;; SampledBits variables...
;;;----------------------------------------------------------

;; If BSC is smaller than the Timer, we should sample a bit...
   movfw BSCHigh
   subwf TMR1H,w	;; Subtract (TMR1H - SBCHigh) leave result in W
   btfsc STATUS,Z	;; If they are the same, we test the low bits...
   goto BS_TEST_LOW_BITS
	
   btfss STATUS,C			;; If TMR1H is lower, we just continue...
   goto BS_DONE				;; We just continue...
   goto BS_TAKE_SAMPLE		;; Otherwise, we take a sample...
   


BS_TEST_LOW_BITS
  movfw BSCLow
  subwf TMR1L,W		;; Subtract (TMR1L - BSCLow)
  btfss STATUS,C	;; If BSCLow is larger than TMR1L, then we just
  goto BS_DONE		;; continue...
  ;; Otherwise, we take a sample...

BS_TAKE_SAMPLE


  rrf SensedValues,w		;; Place the last sampled bit into the C (STATUS,Carry)
  rlf SampledBits1,f		;; Rotate into SampledBits1 and through...
  rlf SampledBits2,f		;; SampledBits2 and...
  rlf SampledBits3,f		;; SampledBits3
  rlf SampledBits4,f
  rlf SampledBits5,f
  rlf SampledBits6,f

;; When we take a sample, we have to add 1 BIT's worth of time to the SBC[High|Low] counter
;; So that we will sample again in the middle of the next bit.
	movfw MBLLow				;; Load low 8 bits of MinimumBitLength
	addwf BSCLow,f				;; Add into BCSLow
	btfsc STATUS,C				;; If we had a cary...
	incf  BSCHigh,f				;; Increment BSCHigh
	movfw MBLHigh				;; Load the high 8 bits...
	addwf BSCHigh,f				;; Add to BSCHigh
	btfsc STATUS,C				;;; If we had an overflow something is seriously wrong!
	goto RESET_AUTOSYNC


;; Check for tracking pattern!
  call DECODE_TRACK

  call DECODE_SEC
;; Note that Decode_sec will increment the "GoodOnes" bit count by 21 every time
;; a valid pattern is detected....


INCR_TOTAL_BIT_COUNT
;; We just took a sample, lets increment our total bit count!
   incfsz TotalBitsLow,f
   goto BS_DONE
   incf TotalBitsHigh,f


;;; -----------------------------------------------------------
;;; - Done checking/taking a sample, lets look for an edge...
;;;;----------------------------------------------------------
BS_DONE

    btfss LastState,0		; Check the last state...
    GOTO AS_LAST_STATE_LOW

;; The previous state was HIGH, we are looking for a low
;; to detect an edge.
AS_LAST_STATE_HIGH
    movlw b'11110000'    ; Looking for high followed by low...
	xorwf SensedValues,w		; Xor W with SensedValues, store in W (because we only care about STATUS.Z)
	btfss STATUS,Z			; Test the Zero bit.  
	GOTO AUTOSYNC_LOOP		;; 

AS_EDGE_DETECTED_LOW
  	bcf LastState,0			; Set the Last State to low (current state)
	goto AS_EDGE_DETECTED


;; The previous state was low, we are looking for low to high
;; to detect an edge.
AS_LAST_STATE_LOW
    movlw b'00001111'		; Looking for lows followed by highs.
	xorwf SensedValues,w	; Xor W with Sensed Values store in W (because we only care about STATUS.Z)
	btfss STATUS,Z			; test the zero bit.
	GOTO AUTOSYNC_LOOP		;; 

AS_EDGE_DETECTED_HIGH
	bsf LastState,0			; Set the last state to High ( current state).
AS_EDGE_DETECTED

	;; If the timer is smaller than the Minimum Bit Length counter, we need to update
	;; the MBL counter.
	;; Because of checks elsewhere, we assume we don't have to worry about an overflow.

;; This didn't work so hot, 65%, went back to old way plus MBL_TOO_LOW_CHECK
;;call UPDATE_MBL_EDGE_DETECTED
;; goto RESET_TIMER

	movfw TMR1H				; Load high 8 bits of timer1 to W
	subwf MBLHigh,w			; MBLHigh - Timer1High, save in W
    btfsc STATUS,Z			; If they are the same, we test the low bits.
	goto AS_ED_TEST_TIMER_LOW ; We test the low bits.
	
	btfss STATUS,C
    goto RESET_TIMER		;; If MBLHigh is lower, we just  reset timer and loop again...




    ; If the Timer is smaller than MBLHigh, we reset MBL with the timer value.

   	movfw TMR1L
	movwf MBLLow
    movfw TMR1H
	movwf MBLHigh

 
;; Make sure it's not TOO low..
  call MBL_TOO_LOW_CHECK
	goto RESET_TIMER 	





AS_ED_TEST_TIMER_LOW		
  movfw TMR1L				;; load low 8 bits of timer1 into W
  subwf MBLLow,w			;; Subtract MBLLow - Timer1Low, save in W
  btfss STATUS,C			;; If Timer1Low is more than MBLLow we just reset the timer
  goto RESET_TIMER			
	movfw TMR1L				;; Otherwise, if Timer1Low is smaller, we reset MBLLow.
	movwf MBLLow

RESET_TIMER
	clrf TMR1L
	clrf TMR1H

SET_SBC_TO_HALF

 
;; Because we detected an edge, we set the SampleBitCounter to half of one bit length.
;; (i.e. we should take our next sample in 1/2 of a bit)
    bcf STATUS,C ;; Make darn sure carry is clear...	
	rrf MBLHigh,w	;; Divide MBLHigh by 2 as we move it into the W register....
	movwf BSCHigh	;; Move into BitSampleCount High.
	rrf MBLLow,w	;; Divide MBLLow by 2 (moving cary bit in) as we move to W register...
	movwf BSCLow    ;; Save to BitSampleCount Low.


;;;;------------------------------
;;;;	Done with Edge Detection, return to top of the loop!
;;;;------------------------------
 goto AUTOSYNC_LOOP		;; Run through the loop again...



;; This code slowly increases the Minimum Bit Lenth measure
;; The idea is to allow us to sync to signals slower than 60hz 
;; (say, 10hz, or 55hz if the sending computer starts to slow down..)


BIT_LENGTH_DRIFT_UP
 ;; Increment the MBLLow (AND MBLHigh on overflow) once every 10 time through the sample loop...

	decf DriftCount,f
	btfss STATUS,Z		;; Reach zero yet?
	goto AS_NEW_SENSE
	movlw d'50'
    movwf DriftCount
	


    incf MBLLow,f	
    btfss STATUS,C 			;; Check for overflow....
 	goto AS_NEW_SENSE
 	incf MBLHigh,f

   ;; If we are at FF, reset to DF
   movlw 0xFF
   xorwf MBLHigh,w
   btfss STATUS,Z
   goto AS_NEW_SENSE
	movlw 0xDF
    movwf MBLLow
    movwf MBLHigh

  return




;; This code, although heavy handed, works well @ 60hz
MBL_TOO_LOW_CHECK

;; If the value is below 0x08, we will reset it to a value of our choosing...
;;	movlw 0x08
;;	subwf MBLHigh,w
;;	btfsc STATUS,C
;;    return


;; Real pattern bit correctness:
;; 0x07de - 0%
;; 0x07ee - 11%
;; 0x07f0 - 15%
;; 0x0800 - 84% - 84.5%
;; 0x0815 - 81.7%
;; 0x0823 - 78%
;; 0x0850 - 82%
;; 0x08dd - 67%
    
    movlw 0x08
    movwf MBLHigh
    movlw 0x00
    movwf MBLLow
    return


UPDATE_MBL_EDGE_DETECTED
 ;; Following code gave 65% accuracy or soabouts...
 ;; ------------------------
 ;;  We want to average only timer measurments which are lower than MBL + 1/2 MBL.
 ;;-------------------------

  bcf STATUS,C
  rlf MBLHigh,w   	;; Find 1/2 MBL High
  addwf MBLHigh,w   ;; Add to MBLHigh
  subwf TMR1H,w    ;; Subtract TMR1H - (MBL + 1/2 MBL)
  btfsc STATUS,C
  return			;; TMR1H is too high, don't average.

  

  ;; Otherwise, Add to the average...
  movfw TMR1L
  addwf MBLAvLow,f
  btfsc STATUS,C
   incf MBLAvHigh,f
  movfw TMR1H
  addwf MBLAvHigh,f

  incf MBLAvCount,f

 ;; If dividing by 8 instead of 4, uncomment extra shifts below...
  movlw 0x08
 
  subwf MBLAvCount,w
  btfss STATUS,Z
  return  ;; Not time to do anything yet...

  ;; OK, we have 4 samples in our MBLAv register(s),  divide them by 4, and move to MBL
  bcf STATUS,C
  rrf MBLAvHigh,f
  rrf MBLAvLow,f

  bcf STATUS,C
  rrf MBLAvHigh,f
  rrf MBLAvLow,f
 
;; Average/divide by 8???
  BCF STATUS,C
  rrf MBLAvHigh,f
  rrf MBLAvLow,f

  bcf STATUS,C
  rrf MBLAvHigh,f
  rrf MBLAvLow,f

  movfw MBLAvHigh
  movwf MBLHigh
  movfw MBLAvLow
  movwf MBLLow

  clrf MBLAvHigh
  clrf MBLAvLow
  clrf MBLAvCount

 
  return

;;-----------------------------------------
;; DECODE_TRACK
;;
;; This code takes a look at the last 10 bits (SampledBits1<7..0>, 2<1,0>) and
;; tries to detect a valid tracking code.
;;
;; A valid tracking code is made up of two framing bits (which are the same)
;; and then six payload bits (3 data bits, each immediately followed by it's inverse)
;; followed by two more framing bits (which are the inverse of the first two framing
;; bits.
;; Eg: The code 0x1 (001) could be sent as: 00 01 01 10 11, or as 11 01 01 10 00
;; (as a matter of fact, if sent more than once the framing bits are shared, so a real
;; payload consists of 10 bits, where the last two bits are shared with the next
;; 8 bit payload, so overall we only have 2 extra bits total added to a bunch of 8 bit
;; payloads.
DECODE_TRACK
  clrf CheckBits
  clrf TEMP
  
;; Do the easy tests first: Make sure the framing bits are correct!

 ; Frame bits SampledBits1,0 and SampledBits2,0  they should be different!
  btfsc SampledBits1,0
   goto _DT_ck_FB0_9s
  btfss SampledBits2,0
   return					;; 0 is clear, 9 is also clear!

  goto _DT_ck_FB1_10

_DT_ck_FB0_9s
 btfsc SampledBits2,0    
   return					;; 0 is set, 9 is also set!


_DT_ck_FB1_10
  btfsc SampledBits1,1
   goto _DT_ck_FB1_10s
 btfss SampledBits2,1	
  return				;; 1 is clear, but 10 is not set!
 goto _DT_ck_frame

_DT_ck_FB1_10s
  btfsc SampledBits2,1
    return				;; 1 is set, but 10 is not set!


;; Ok, the 0-9 and 1-10 bit checks work, lets compare 0-1 and make
;; sure that they are the same (if they are, 9-10 are also the same
;; due to the previous checks.)

_DT_ck_frame
  btfsc SampledBits1,0
   goto _DT_ck_frame_set
  btfsc SampledBits1,1
   return				;; bit zero is clear, but bit 1 isn't.

  goto _DT_ck_data

_DT_ck_frame_set
  btfss SampledBits1,1	
    return				;; bit zero is set, but bit 1 isn't set.

_DT_ck_data
;; Ok, all of the framing bits check out...make sure that the data is
;; appropriately inverted. To do this we take every other bit of data and
;; move it into Accumulator1 and Accumulator2, then compare them to one another with an XOR.

  bcf STATUS,C
  clrf Accumulator1
	;; Accumulator 1 has all zeros.
  clrf Accumulator2
  decf Accumulator2,f
    ;; Accumulator2 has all ones.


  movfw SampledBits1
  movwf TEMP

  ; remove framing bits...
  rrf TEMP,f
  rrf TEMP,f

  rrf TEMP,f
  rlf Accumulator2,f ;; Check bit 2
  rrf TEMP,f
  rlf Accumulator1,f ;; data bit 2
  rrf TEMP,f
  rlf Accumulator2,f ;; CheckBit 1
  rrf TEMP,f
  rlf Accumulator1,f ;; data Bit 1
  rrf TEMP,f
  rlf Accumulator2,f ;; Check Bit 0
  rrf TEMP,f
  rlf Accumulator1,f ;; Data Bit 0

  ;; Invert Accumulator1
  movlw 0xFF;
  xorwf Accumulator1,f

  ;; Test to see if they are the same as their inverted checkbits!
  movfw Accumulator2
  xorwf Accumulator1,f

 btfss STATUS,Z
   return			;; Data is not an accurate inverse of it's checkbits.
  

;; Ok, the data passed all of the checks we could throw at it. It just may be a
;; valid tracking code. Lets report it back to the computer!

;; Save the data into our bytes to send. (NOTE! We only save the 10 data bits.
 ;; We also add a 4 bit (0-15) number representing which sensor we got the value off
 ;; of. (Currently, we only have one sensor, number zero...)
 ;; We also add 2 bits indicating what type of tracking pattern it was (10 - 3 bit track)
 ;; 10 3210 9876543210
 ;; MM-SSSS-DDDDDDDDDD
 ;; | High    | Low   |

 clrf LowByteToSend
 clrf HighByteToSend
 ;; for testing
 btfsc SampledBits1,3	; Data bit 1
   bsf LowByteToSend,0
 btfsc SampledBits1,5	; Data bit 2
   bsf LowByteToSend,1
 btfsc SampledBits1,7	; Data bit 3
   bsf LowByteToSend,2	

 ;; Which sensor was it? It was sensor 0, so all 4  bits are zero, so we don't set anything!

 ;; Which tracking code was it? It was code 10 (3 bit track) so we set bit 7
 bsf HighByteToSend,7


;; Send the value over the radio!
 call XMIT


;; FOR TESTING - update the count of how many bits worked correctly.
 ;; we increment our good  bits count by 8 (as 2 of the 10 bits are shared)
    movlw d'8'			; Decimal 8 			
    addwf GoodOnesLow,f
    btfsc STATUS,C
    incf GoodOnesHigh,f

  return
  
;;-----------------------------------------
;; DECODE_SEC
;;
;; This code takes a look at the SampledBits1,2,3 bytes, 
;; looking specificly at the latest 21 bits sampled. It attempts to
;; perform a checksum opertion on them (SEC) and if it appears to be a valid
;; bitsequence, it will send the data out over the radio link.
;;
;; It should be called after each bit is sampled.
;; Uses, TEMP,W, CheckBits.
DECODE_SEC
  clrf CheckBits
  clrf TEMP

;; Decode Checkbit number 1 ... (parity of bit positions 3,5,7,9,11,13,15,17,19,21)
  btfsc SampledBits1,0		; Bit Position 21
  incf TEMP,f
  btfsc SampledBits1,2		; Bit Position 19
  incf TEMP,f
  btfsc SampledBits1,4		; Bit Position 17
  incf TEMP,f
  btfsc SampledBits1,6		; Bit Position 15
  incf TEMP,f
  btfsc SampledBits2,0		; Bit Position 13
  incf TEMP,f
  btfsc SampledBits2,2		; Bit Position 11
  incf TEMP,f
  btfsc SampledBits2,4		; Bit Position 9
  incf TEMP,f
  btfsc SampledBits2,6		; Bit Position 7
  incf TEMP,f
  btfsc	SampledBits3,0		; Bit Position 5
  incf TEMP,f
  btfsc SampledBits3,2		; Bit Position 3
  incf TEMP,f

  ;; Move bit into checkbits through carry.
  rrf TEMP,f
  rlf CheckBits,f

;; Decode Checkbit number 2 ... (parity of bit positions 3, 6,7,  10,11, 14,15, 18,19)
  clrf TEMP
 
  btfsc SampledBits1,2		; Bit Position 19
  incf TEMP,f
  btfsc SampledBits1,3		; Bit Position 18
  incf TEMP,f

  btfsc SampledBits1,6		; Bit Position 15
  incf TEMP,f
  btfsc SampledBits1,7		; Bit Position 14
  incf TEMP,f

  btfsc SampledBits2,2		; Bit position 11
  incf TEMP,f
  btfsc SampledBits2,3		; Bit Position 10
  incf TEMP,f

  btfsc SampledBits2,6		; Bit Position 7
  incf TEMP,f
  btfsc SampledBits2,7		; Bit Position 6
  incf TEMP,f

  btfsc SampledBits3,2		; Bit Position 3
  incf TEMP,f
  
  ; Move Bit into CheckBits through Carry
  rrf TEMP,f
  rlf CheckBits,f


 ;; Decode Checkbit number 3 ... (parity of bit positions 5-7, 12-15, 20,21
  clrf TEMP
  
  btfsc SampledBits1,0		; Bit Position 21
  incf TEMP,f
  btfsc SampledBits1,1		; Bit Position 20 
  incf TEMP,f
  
  btfsc SampledBits1,6      ; Bit Position 15
  incf TEMP,f
  btfsc SampledBits1,7		; Bit Position 14
  incf TEMP,f
  btfsc SampledBits2,0		; Bit Position 13
  incf TEMP,f
  btfsc SampledBits2,1		; Bit Position 12
  incf TEMP,f

  btfsc SampledBits2, 6		; Bit Position 7
  incf TEMP,f
  btfsc SampledBits2, 7		; Bit Position 6
  incf TEMP,f
  btfsc SampledBits3, 0		; Bit Position 5
  incf TEMP,f

  ;; Move Bit into CheckBits through Carry
  rrf TEMP,f
  rlf CheckBits,f

  

  ;; Decode Checkbit number 4... (parity of bit positions 9-15)
  clrf TEMP
  btfsc SampledBits1,6   	; Bit position 15
  incf TEMP,f
  btfsc SampledBits1,7		; Bit Position 14
  incf TEMP,f
  btfsc SampledBits2,0		; Bit Position 13
  incf TEMP,f
  btfsc	SampledBits2,1		; Bit Postion 12
  incf TEMP,f
  btfsc SampledBits2,2		; Bit Postion 11
  incf TEMP,f
  btfsc SampledBits2,3		; Bit Position 10
  incf TEMP,f
  btfsc SampledBits2,4		; Bit Position 9
  incf TEMP,f

  ;; Move Bit into CheckBits through carry.
  rrf TEMP,f
  rlf CheckBits,f
  
  ;; Decode checkbit number 5... (parity of bit positions 17-21)
  clrf TEMP
  ;; Count the number of 1 bits...
  btfsc SampledBits1,0  	; Bit position 21
  incf TEMP,f
  btfsc SampledBits1,1  	; Bit position 20
  incf TEMP,f
  btfsc SampledBits1,2  	; Bit position  19
  incf TEMP,f
  btfsc SampledBits1,3  	; Bit position  18
  incf TEMP,f
  btfsc SampledBits1,4   	; Bit Position 17
  incf TEMP,f

  ;; Move bit into CheckBits through Carry.
  rrf TEMP,f
  rlf CheckBits,f

  ;; Now, move the received checkbits into TEMP
  ;;
  clrf TEMP
  btfsc SampledBits3,4
  bsf TEMP,4
  btfsc SampledBits3,3
  bsf TEMP,3
  btfsc SampledBits3,1
  bsf TEMP,2
  btfsc SampledBits2,5
  bsf TEMP,1
  btfsc SampledBits1,5
  bsf TEMP,0

 ;; Now, check the calculated checkbits vs the actual checkbits.
 movfw TEMP
 xorwf CheckBits,w		 ; Leave result in W (indicates position of bit error...)
 btfss STATUS,Z				;; If we have an exact match, there are no errors (or more than 3....)
  return

;; Technically, nothing but zero's is a valid bit sequence....but I don't like this sequence
;; (as it looks like the sensor not seeing anything...) so I'm going to discriminate against it 
;; if it appears in the SampledBits1 byte (as that byte should always have a 0x15 in it... (see below)
  clrw 
  xorwf SampledBits1,w
  btfsc STATUS,Z
  return

;; Our projector system sends a A8 at the end of the data, which (when reversed) looks to us like:
; 0x15 in the end of our data. We AND the last byte by 0x1F to isolate it and then
;; test for it. This characteristic gives us much greater accuracy!

  movlw 0x15
  movwf TEMP
  movlw 0x1F 			;; Why, oh why, is movlw 0x1f different from 0x1F????
  andwf SampledBits1,w
  xorwf TEMP,w
  btfss STATUS,Z
  return
;; Ok, we got a live one!!!

 ;; All of the checkbits agree!
 ;; Save the data into our bytes to send. (NOTE! We only save the 10 data bits.
 ;; We also add a 4 bit (0-15) number representing which sensor we got the value off
 ;; of. (Currently, we only have one sensor, number zero...)
 ;; We also add 2 bits indicating what type of tracking pattern it was (01 - 10 bit track. 10-3 bit track)
 ;; 10 3210 98765432
 ;; MM-SSSS-DDDDDDDDDD
 ;; | High    | Low   |

 clrf LowByteToSend
 clrf HighByteToSend
 
 btfsc SampledBits3,2	; Data bit 1
   bsf LowByteToSend,0
 btfsc SampledBits3,0	; Data bit 2
   bsf LowByteToSend,1
 btfsc SampledBits2,7	; Data bit 3
   bsf LowByteToSend,2	
 btfsc SampledBits2,6   ; Data bit 4
   bsf LowByteToSend,3
 btfsc SampledBits2,4	; Data bit 5
   bsf LowByteToSend,4
 btfsc SampledBits2,3  	; Data bit 6
   bsf LowByteToSend,5
 btfsc SampledBits2,2	; Data Bit 7
   bsf LowByteToSend,6	
 btfsc SampledBits2,1	; Data bit 8
  bsf LowByteToSend,7	
 btfsc SampledBits2,0	; Data bit 9
  bsf HighByteToSend,0
 btfsc SampledBits1,7	; Data bit 10
  bsf HighByteToSend,1

 ;; Which sensor was it? It was sensor 0, so all 4  bits are zero, so we don't set anything!

 ;; Which tracking code was it? It ws code 01 (10 bit track) so we set bit 6 (or 7 if you count from 1)
 bsf HighByteToSend,6

 	

;; FOR TESTING - update the count of how many bits worked correctly.
;;Increment by 21!
 ;; we increment our good  bits count by 21
    movlw 0x15			; Decimal 21 			
    addwf GoodOnesLow,f
    btfsc STATUS,C
    incf GoodOnesHigh,f

  incf ProbeCounter,f 

;; Send the value over the radio!
 call XMIT

  return

;; Because we decoded a correct signal, we should save it and go sleep and things... FOR TESTING
  ;;goto SAVE_DEBUG_STATE_AND_SLEEP






;----------------------------------
; fill in transmission buffer
;----------------------------------

XMIT

       
	;; If we are currently transmitting from the last time,
    ;; wait here until we are done...
FT_LOOP_UNTIL_DONE
    btfsc TX_STATUS,0
     goto FT_LOOP_UNTIL_DONE

        movlw   0x73            ; send Serial number  
        movwf   CSR0

        movfw   FuncBits        ; send Function Bits - We may not need to send this either...
        movwf   CSR1

                                ; send 16-bit Counter
                                ;
                                ; send analog value (this is modified from the fixed 
                                ;   KeeLoq(r) protocol; these two fields are normally 
                                ;   used for 16-bit counter value)

		;; Our payload:
        movfw   HighByteToSend          ; First byte.
        movwf   CSR2

        movfw   LowByteToSend			; Second byte.
        movwf   CSR3



;; For some reason we must still send CSR4 for the receive to detect the signal...

        movlw   0x56            ; send 32-bit serial number 1 (ignored by receiver)
        movwf   CSR4

 
;; Start the interrupt driven Transmitter!

 movlw D'1' ;; State = 1 TX_START_PREAMBLE
 movwf TX_STATE
 bsf TX_STATUS,0	;; Start transmitting.

 return 				;; return to whoever called us!










;----------------------------------------------------------------------
; TX_INTERRUPT
;
;  This is the main interrupt handler for Interupt based transmission.
;  Depending upon the TX_STATE variable it will shuffle controll
;  off to the various parts of the interrupt driven TX state machine.

TX_INTERRUPT
 ;  bcf TX_STATUS,0
 ;  return
    movlw high TX_STATE_MACHINE_START
    movwf PCLATH
    movf  TX_STATE,W
    andlw 0x0F;			;; May need to CHANGE ME if more than 16 states (below)!
    addlw low TX_STATE_MACHINE_START
    btfsc STATUS,C
    incf PCLATH,F
    movwf PCL

							;; load PC with computed goto value.

TX_STATE_MACHINE_START
 goto TX_RADIO_INACTIVE				;0
 goto TX_PREAMBLE_START				;1
 goto TX_PREAMBLE_ON				;2
 goto TX_PREAMBLE_OFF				;3
 goto TX_SYNC_TIME_START			;4
 goto TX_SYNC_TIME_WAIT				;5	
 goto TX_NEXT_BYTE					;6
 goto TX_NEXT_BIT					;7
 goto TX_TRANSITION_ONE				;8
 goto TX_END_BIT					;9
 goto TX_GUARD_TIME_START			;10	
 goto TX_GUARD_TIME_WAIT			;11
 goto TX_FINISHED					;12


;------------------
; TX_RADIO_INACTIVE
; 
; We are not currently transmitting. As a matter of fact, we shouldn't reach
; this state if the TX_STATUS bit is set correctly...
TX_RADIO_INACTIVE
  ;; We really shouldn't get to this position...
  bcf TX_STATUS,0
  clrf TMR0
  return
;-----------------
; TX_PREAMBLE_START
;
; This is the PREAMBLE Init. So we load TPREAMBLE into the TX_COUNTER
; (number of interrupt periods we should toggle) and then increment the 
; state to TX_PREAMBLE_ON 
TX_PREAMBLE_START

  bsf     RFENA           ; Enable Transmitter

  movlw  PREAMB
  movwf TX_COUNTER
  incf TX_STATE,F
  return

;------------------
; TX_PREAMBLE_ON
;
; Turn on the transmitter for one bit, then advance to TX_PREAMBLE_OFF
;
TX_PREAMBLE_ON
   bsf     TXD 				; ON
   incf TX_STATE,F			; Move on to TX_PREAMBLE_OFF next...
   return

;--------------------
; TX_PREAMBLE_OFF
;
; Turn the transmitter off for one bit. Then, Decrement the TX_COUNTER
; and (if not zero) decrement back to TX_PREAMBLE_ON.
; Basically we are flashing an on-off pattern PREAMB times.  
TX_PREAMBLE_OFF
 bcf TXD					; OFF
 decf TX_COUNTER,F
 btfsc STATUS,Z
 goto TX_PREAMBLE_OFF_DONE
 decf TX_STATE,F			; Not done yet, go back a state to TX_PREAMBLE_ON
 return

;; We are done with the preamble, radio is turned off, go to the TX_SYNC_TIME_START state
TX_PREAMBLE_OFF_DONE

 incf TX_STATE,F
 return

;-----------------
; TX_SYNC_TIME_START
;
; This is the SYNC time. So we load TSYNC into the TX_COUNTER
; (number of interrupt periods we should wait) and then increment the state
; to TX_SYNC_TIME_WAIT. Note that the radio is turned off now (by previous state)
TX_SYNC_TIME_START
  movlw  TSYNC
  movwf TX_COUNTER
  incf TX_STATE,F
  return

;------------------
;TX_SYNC_TIME_WAIT
;
; We are currently waiting on the SYNC time. One interrupt has
; occured, so we decrement the TX_COUNTER. If it reaches zero,
; we are done waiting, so we change the STATE to TX_FINISHED.
TX_SYNC_TIME_WAIT
 decf TX_COUNTER,F
 btfss STATUS,Z
 return

 ;; We are done with the Sync, now we get ready to transmit!
 
;  movlw D'8'		; Send 64 bits in total, 8 bytes.
  movlw D'5'		;; We are sending CSR0-CSR4 (5 bytes) total....
  movwf TX_COUNTER

 ;; Load our indirection value for FSR into TX_FSR
 movlw CSR0
 movwf TX_FSR

 
 incf TX_STATE,F		;; Move on to TX_NEXT_BYTE
 return

;-------------------
; TX_NEXT_BYTE
;
; Get the next byte ready to send.
;
TX_NEXT_BYTE

  ;; We turn on the transmitter right now! 
  ;; (I.E. this code runs inside of the transmition of the first "ON" phase.
  ;;   as we roll over into TX_NEXT_BIT.
  bsf TXD

 ;; Save the FSR register (so we don't mess up people outside of the ISR)
 ;; At the same time, move TX_FSR into FSR.
  SWAPWF FSR			; Note that the macro makes this 9 instructions total!
  SWAPWF TX_FSR		; 3 for each swap.
  SWAPWF FSR

 ; Move the current byte to send into TX_BYTE_TEMP.
  movfw INDF
  movwf TX_BYTE_TEMP		

 ;; Swap the FSR back out.
  SWAPWF FSR
  SWAPWF TX_FSR
  SWAPWF FSR
 
  movlw D'8'			; 8 bits in a byte.
  movwf TX_BIT_COUNTER


  incf TX_STATE,F	; We are moving into TX_NEXT_BIT, so we have to increment this,
					;; Even if we don't loop through another interupt.
  ;return

	;; We now move on to TX_NEXT_BIT. Note that we do NOT pause for another interrupt here!
    ;; So, the transmitter turns on almost immediately (with the briefest of pauses to
    ;; increment to the next byte...


;-----------------
; TX_NEXT_BIT
;
; Determine the next bit to send, and set the TX_STATUS.1 bit appropriately.
; Then, turn on the transmitter and  advance to TX_TRANSITION_ONE
;
TX_NEXT_BIT
  ;; First. Turn on the transmitter (to get it on as early as possible.)
  ;; This only matters if we are being called directly (as opposed to falling
  ;; through from TX_NEXT_BYTE.
  bsf TXD

  bcf TX_STATUS,1			;; Bit that holds the value of the current bit to save...
							;; We start off assuming our bit is a zero...

  rrf TX_BYTE_TEMP,F		;; Move the bit to send into carry.
  btfsc STATUS,C			;; test it...
  bsf TX_STATUS,1			;; If our bit is a one, set TX_STATUS.1


  incf TX_STATE,F		;; Move to TX_TRANSITION_ONE
  return


;; How does this next part work you ask? Easy...
;ZERO            
; 
;   +---+---+   +--
;   |       |   |
;---+       +---+
;   |  2Te   Te |
;
; If we are sending a zero (clear) we leave the transmitter on until TX_END_BIT.

;ONE
;              
;   +---+       +--
;   |   |       |
;---+   +---+---+
;   | Te   2Te  |
; 
; If we are sending a one (set) we turn the transmitter off early, at TX_TRANSITION_ONE
;
; The receiver checks the time on vs the time off and figures out what bit we are sending... easy as pie.
;
;
; To Clarify how the states of the machine interact with the above pattern, look at this:
;
;   +---+       +--
;   |   |       |
;---+   +---+---+
;   1   2   3   1  2....
;   +---+---+   +--
;   |       |   |
;---+       +---+
;   |  2Te   Te |
;
; TX_NEXT_BYTE and TX_NEXT_BIT run at point 1.
; TX_TRANSITION_ONE is at position 2.
; TX_END_BIT runs at point 3.
;
; So, after TX_NEXT_BYTE and TX_NEXT_BIT get done, the transmitter is turned on until
; at least point 2. At point 2, TX_TRANSITION_ONE gets ran. If we are transmitting a one, 
; we turn the transmitter off here. (If a zero, we don't.)
;
; Then, at point 3 TX_END_BIT gets called. It turns the transmitter off (again in the case of a one)
; and then sets things up to jump back to either TX_NEXT_BIT or TX_NEXT_BYTE (which happen at point 1
; of the next bit or byte. And the cycle continues.

;----------------
; TX_TRANSITION_ONE
;
; We are not at point 2 in the above ascii art figures...
; If we are sending a ONE, we turn off the transmitter. If we are sending
; a zero, we leave it on (to be turned off at TRANSITION_TWO).
;
TX_TRANSITION_ONE

  btfsc TX_STATUS,1		;; Test to see what bit we are supposed to be sending.
	bcf TXD				;; If it's a 1 (SET) we turn off the transmitter.


  incf TX_STATE,F		; Move on to TX_END_BIT
  return

;---------------------
; TX_END_BIT
;
; We are now at point 3 in the above ascii art pictures..
;
; We decrement the bit sent count. If it's not zero, we go back to
; TX_NEXT_BIT. If it is zero, we are done with this byte, so we decremnt
; TX_COUNTER. If TX_COUNTER is not zero, we inc the TX_FSR and jump to TX_NEXT_BYTE. If it is zero,
; it means we are done, and we jump to TX_GUARD_TIME_START.
TX_END_BIT

  ;; Doesn't really matter what the bit is set to, once we get here we want
  ;; the transmitter off no matter what...
  bcf TXD

  decf TX_BIT_COUNTER,F
  btfss STATUS,Z
  goto _TX_END_BIT_NEXT_BIT ;; Bit counter is not zero, so we do the next bit.
 
  ;; Bit counter was zero, so we are done with the current byte.
 
  decf TX_COUNTER,F
  btfsc STATUS,Z			;; Check to see if we are all done..
  goto _TX_END_BIT_DONE		;; Ok, we are done sending all the bytes 

 ;; Not all done, so we go back to TX_NEXT_BYTE
_TX_END_BIT_NEXT_BYTE
  incf TX_FSR,F			;; Move on to the next byte!

  movlw D'6'			; TX_NEXT_BYTE
  movwf TX_STATE
  return

 ;; We need to send the next bit, so we jump to TX_NEXT_BIT
_TX_END_BIT_NEXT_BIT
 movlw D'7' 		; TX_NEXT_BIT
 movwf TX_STATE
 return

 ;; We are all done sending bits, so we increment to TX_GUARD_TIME_START
_TX_END_BIT_DONE
 incf TX_STATE,F
 return




;-----------------
; TX_GUARD_TIME_START
;
; It is time to start the guard time. So we load TGUARD into the TX_COUNTER
; (number of interrupt periods we should wait) and then increment the state
; to TX_GUARD_TIME_WAIT. Note that the radio is turned off now (by previous state)
TX_GUARD_TIME_START
  movlw  TGUARD
  movwf TX_COUNTER
  incf TX_STATE,F
  return

;------------------
;TX_GUARD_TIME_WAIT
;
; We are currently waiting on the guard time. One interrupt has
; occured, so we decrement the TX_COUNTER. If it reaches zero,
; we are done waiting, so we increment the STATE to TX_FINISHED.
TX_GUARD_TIME_WAIT
 decf TX_COUNTER,F
 btfss STATUS,Z
 return
 ;; We are done, so increment state!
 incf TX_STATE,F
 return

;--------------------
; TX_FINISHED
;
; We are done with this transmission. Do all of the cleanup, and then
; clear the state register back to TX_RADIO_INACTIVE.
TX_FINISHED
  bcf     RFENA              ; Shut down the Transmitter
  clrf TX_STATE				;; Next state is TX_RADIO_INACTIVE
  bcf TX_STATUS,0			;; Clear the "currently transmitting" flag
  return









;----------------------------------------------------------------------
; Data EEPROM Memory (Section 8.0)
;
; PIC12F629/675 devices have 128 bytes of data EEPROM with address
; range 0x00 to 0x7F.

        ; Initialize first few Data EEPROM Memory locations to 0xFF.

        ORG 0x2100
        DE  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF


;----------------------------------------------------------------------
; Calibrating the Internal Oscillator (Section 9.2.5.1)
; Oscillator Calibration Register (OSCCAL) (Section 2.2.2.7)
;
; The below statements are placed here so that the program can be
; simulated with MPLAB SIM.  The programmer (PICkit or PROMATE II)
; will save the actual OSCCAL value in the device and restore it.
; The value below WILL NOT be programmed into the device.

        org     0x3ff
        retlw    0x80

;----------------------------------------------------------------------
        end                         ; end of program directive
;----------------------------------------------------------------------
