
/* LinearTracker.cpp
 *  - Defines a tracker with a Linear motion model for the sensor. 
 *  We use the last two sensed values to determine if the sensor is
 *  in motion, and if so, what vector it is moving along.
 */
#include <iostream>	// for NULL
#include "LinearTracker.h"




LinearTracker::LinearTracker(float maxX, float maxY) : Tracker(maxX, maxY)
{
 // We don't do anything different from the default, this is just here so
 // that the default constructer gets called with the appropriate arguments.
 
}

  // When the system is ready to put up a new tracker, it will
  // ask the Tracker object where it should be placed.
  // The "timestep" is the current timestep, while the X,Y,Size
  // pointers are used as output variables to return the answer.
  //
  // The return value is a measure of how well we think we are doing.
  // This number drops to zero as we loose the sensor, and goes towards
  // 1.0 as we track it.
  //
float LinearTracker::getTrackerPosition(int timestep, float *X, float *Y, float *size)
{
	int numHist = getNumHistory();

	if (numHist == 0)
	{
		return(0.0);	// No confidence to make any predictions.
	}
	else if (numHist == 1)
	{

	  // With one entry, we act like the Simple (static) tracker,
	  // and just give out the last observation as the new location. 
	  h_ent * last = getHistory(0);
	  *X = last->x;
	  *Y = last->y;
	  *size = DEFAULT_TRACKER_SIZE;
	  // Figure out how long it's  been since we last saw something.
	  int difference = timestep - last->timestep;

 	  // If it's been over a two seconds, we've lost it!
	  if (difference > 2500)
	  	return(0.0);

	  // If it's smaller than 2 seconds, return a value based upon
	  // the time.
	  return( ((float)(2500 - difference)) / 2500.0);
	}
	else if (numHist > 1)
	{
	  h_ent* last = getHistory(0);
	  h_ent* prev  = getHistory(1);

	  float d_x = last->x - prev->x;
	  float d_y = last->y - prev->y;
	  float speed = (float) (last->timestep - prev->timestep);

          float timeSinceLast = (float)(timestep - last->timestep);

	  float d_shift = timeSinceLast / speed; 

	  // Only move if we have relatively recient data and the
	  // two pints we are using to calculate are close together!
	  if (timeSinceLast < 400 && speed < 300)
	  {
	   *X = last->x + (d_x * d_shift);
	   *Y = last->y + (d_y * d_shift);
	  }
	  else
	  {
	   // If we don't have a good motion model, act like the static
	   // tracker and just position the tracker where the last
	   // observation was made.
	   *X = last->x;
	   *Y = last->y;
	  }

	  checkBounds(X,Y);

          //printf("dx: %f, Speed: %f, dx_shift: %f\n",d_x,speed,d_shift);

	  
	// If it's been over a 2.5 seconds, we've lost it!
	if (timeSinceLast > 2500)
	{
		return(0.0);
	}

	// If it's smaller than 2.5 seconds, return a value based upon
	// the time.
	return( ((float)(2500 - timeSinceLast)) / 2500.0);

	}

}



